SynchroConcurrentActivity subclass: #CountersActivity
	instanceVariableNames: 'receivedDictionary acceptedDictionary completedDictionary '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Counters'!
CountersActivity comment:
'This class has been initially designed and implemented by the DESS class at Universite de Nantes, advised by Jean Bezivin and Olivier Roux. This is a revised version.'!


!CountersActivity methodsFor: 'initialize'!

privateInitialize
	"Create the three synchronization counters dictionaries."

	super privateInitialize.
	receivedDictionary := IdentityDictionary new.
	acceptedDictionary := IdentityDictionary new.
	completedDictionary := IdentityDictionary new.
	oself class allScriptSelectors
		do: [:selector |
			receivedDictionary at: selector put: 0.
			acceptedDictionary at: selector put: 0.
			completedDictionary at: selector put: 0]! !

!CountersActivity methodsFor: 'message handling'!

acceptMessage: aMessage
	"Accept a message by launching a behavior sub process to consume the message."

	self startSubProcessFor: aMessage!

isAcceptableMessage: aMessage
	"Checking next message by evaluating its guard."
	"Note that checking the guard AND incrementing the accept counter
	(if the guard evaluates to true) must be within the same atomic block.
	(Consequently we don't make use of the accept generic synchro event
	to increment the accept counter.)"

	^synchroMutualExclusion critical:
		[(self satisfyGuardSelector: aMessage selector arguments: aMessage arguments)
			ifTrue:
				[self incrAccepted: aMessage selector.
				true]
			ifFalse:
				[false]]! !

!CountersActivity methodsFor: 'guard testing'!

findGuardSelector: selector
	"A guard is simply associated to a selector with the prefix guardOf.
	For instance the guard of method put: is the method guardOfPut:."

	^('guardOF' , selector) asSymbol!

satisfyGuardSelector: selector arguments: argumentsArray
	"Evaluate and return the guard method."

	^self perform: (self findGuardSelector: selector) withArguments: argumentsArray! !

!CountersActivity methodsFor: 'synchro events'!

synchroEventComplete: aMessage
	super synchroEventComplete: aMessage.
	self incrCompleted: aMessage selector!

synchroEventReceive: aMessage
	super synchroEventReceive: aMessage.
	self incrReceived: aMessage selector! !

!CountersActivity methodsFor: 'counters accessing'!

accepted: selector
	"Number of activations accepted for message selector."

	^acceptedDictionary at: selector!

completed: selector
	"Number of activations completed for message selector."

	^completedDictionary at: selector!

current: selector
	"Number of current activations for message selector."

	^(self accepted: selector) - (self completed: selector)!

pending: selector
	"Number of pending activations for message selector."

	^(self received: selector) - (self accepted: selector)!

received: selector
	"Number of requests received for message selector."

	^receivedDictionary at: selector! !

!CountersActivity methodsFor: 'counters updating'!

incrAccepted: selector
	"Increment the number of activations accepted for message selector."

	acceptedDictionary at: selector
					put: (acceptedDictionary at: selector) + 1!

incrCompleted: selector
	"Increment the number of activations completed for message selector."

	completedDictionary at: selector
					put: (completedDictionary at: selector) + 1!

incrReceived: selector
	"Increment the number of messages received for message selector."

	receivedDictionary at: selector
					put: (receivedDictionary at: selector) + 1! !

ImplicitReplyObject subclass: #CountersObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Counters'!


!CountersObject methodsFor: 'default classes'!

activityClass
	"Only purpose is to dispatch onto activity class CountersActivity."

	^CountersActivity! !
