BoundedBuffer subclass: #OCoreBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-OCore-Ex'!


!OCoreBoundedBuffer methodsFor: 'script'!

getAndReplyTo: r
	"If empty, trigger the buffer empty user defined event"

	self isEmpty
		ifTrue: [self activity emptyEvent]
		ifFalse: [super getAndReplyTo: r]!

put: item
	"If full, trigger the buffer full user defined event"

	self isFull
		ifTrue: [self activity fullEvent]
		ifFalse: [super put: item]! !

!OCoreBoundedBuffer methodsFor: 'default classes'!

activityClass
	"OCore-based associated activity."

	^BoundedBufferOCoreActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OCoreBoundedBuffer class
	instanceVariableNames: ''!


!OCoreBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	super exampleSize: maxSize numberItems: numberItems speedRatio: ratio! !

OCoreBoundedBuffer subclass: #OCoreGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-OCore-Ex'!


!OCoreGet2BoundedBuffer methodsFor: 'script'!

get2AndReplyTo: r
	"If less than two elements, trigger the buffer empty user defined event"

	self hasAtLeastTwoElements not
		ifTrue: [self activity suspendGet2Event]
		ifFalse: [self getAndReplyTo: r;
					getAndReplyTo: r]! !

!OCoreGet2BoundedBuffer methodsFor: 'state predicates'!

hasAtLeastTwoElements
	^contents size >= 2! !

!OCoreGet2BoundedBuffer methodsFor: 'default classes'!

activityClass
	"OCore-based associated activity."

	^Get2BoundedBufferOCoreActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OCoreGet2BoundedBuffer class
	instanceVariableNames: ''!


!OCoreGet2BoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGet2: numberItems! !

OCoreBoundedBuffer subclass: #OCoreGgetBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-OCore-Ex'!


!OCoreGgetBoundedBuffer methodsFor: 'script'!

ggetAndReplyTo: r
	"If less than two elements, trigger the buffer empty user defined event"

	self activity afterPut
		ifTrue: [self activity ggetAfterPutEvent]
		ifFalse: [self getAndReplyTo: r]! !

!OCoreGgetBoundedBuffer methodsFor: 'default classes'!

activityClass
	"OCore-based associated activity."

	^GgetBoundedBufferOCoreActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

OCoreGgetBoundedBuffer class
	instanceVariableNames: ''!


!OCoreGgetBoundedBuffer class methodsFor: 'example'!

exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGgetSize: 3 numberItems: 8 speedRatio: 1"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGget: numberItems! !

OCoreActivity subclass: #BoundedBufferOCoreActivity
	instanceVariableNames: 'getQueue putQueue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-OCore-Ex'!


!BoundedBufferOCoreActivity methodsFor: 'initialize'!

privateInitialize
	super privateInitialize.
	getQueue := OrderedCollection new.
	putQueue := OrderedCollection new! !

!BoundedBufferOCoreActivity methodsFor: 'event handlers'!

emptyEvent
	getQueue add: currentMessage!

fullEvent
	putQueue add: currentMessage! !

!BoundedBufferOCoreActivity methodsFor: 'events'!

kernelEventComplete: aMessage
	"Compute one suspended request if condition is fullfilled."

	super kernelEventComplete: aMessage.
	(putQueue isEmpty or: [oself isFull])
		ifFalse:
			[self acceptMessage: putQueue removeFirst]
		ifTrue:
			[(getQueue isEmpty or: [oself isEmpty])
				ifFalse:
					[self acceptMessage: getQueue removeFirst]]! !

BoundedBufferOCoreActivity subclass: #GgetBoundedBufferOCoreActivity
	instanceVariableNames: 'afterPut ggetQueue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-OCore-Ex'!


!GgetBoundedBufferOCoreActivity methodsFor: 'initialize'!

privateInitialize
	super privateInitialize.
	ggetQueue := OrderedCollection new! !

!GgetBoundedBufferOCoreActivity methodsFor: 'accessing'!

afterPut
	^afterPut! !

!GgetBoundedBufferOCoreActivity methodsFor: 'event handlers'!

ggetAfterPutEvent
	ggetQueue add: currentMessage! !

!GgetBoundedBufferOCoreActivity methodsFor: 'events'!

kernelEventComplete: aMessage
	"Compute one suspended request if condition is fullfilled."
	"May resume a suspended gget after resuming a suspended get.
	But the problem of fairness between get and gget requests is not very well handled."

	super kernelEventComplete: aMessage.
	afterPut := aMessage selector = #put:.
	(ggetQueue isEmpty or: [afterPut])
		ifFalse: [self acceptMessage: ggetQueue removeFirst]! !

BoundedBufferOCoreActivity subclass: #Get2BoundedBufferOCoreActivity
	instanceVariableNames: 'get2Queue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-OCore-Ex'!


!Get2BoundedBufferOCoreActivity methodsFor: 'initialize'!

privateInitialize
	super privateInitialize.
	get2Queue := OrderedCollection new! !

!Get2BoundedBufferOCoreActivity methodsFor: 'event handlers'!

suspendGet2Event
	get2Queue add: currentMessage! !

!Get2BoundedBufferOCoreActivity methodsFor: 'events'!

kernelEventComplete: aMessage
	"Compute one suspended request if condition is fullfilled."
	"May resume a suspended get2 after resuming a suspended get.
	But the problem of fairness between get and get2 requests is not very well handled."

	super kernelEventComplete: aMessage.
	(get2Queue isEmpty or: [oself hasAtLeastTwoElements])
		ifFalse: [self acceptMessage: get2Queue removeFirst]! !
